/*
 * Copyright (C) 2022 favdb
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <http://www.gnu.org/licenses/>.
 */
package storybook.toolkit.swing;

import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Graphics2D;
import java.awt.Rectangle;
import java.awt.RenderingHints;
import java.awt.geom.AffineTransform;
import java.awt.geom.Area;
import java.awt.image.BufferedImage;
import java.beans.PropertyChangeEvent;
import java.beans.PropertyChangeListener;
import javax.swing.Action;
import javax.swing.Icon;
import javax.swing.JButton;
import javax.swing.JComponent;
import javax.swing.event.ChangeEvent;
import javax.swing.event.ChangeListener;

/**
 *
 * @author favdb
 */
public class RotatingButton extends JButton {

	private static final long serialVersionUID = 1L;

	protected static final RenderingHints qualityHints;

	static {
		qualityHints = new RenderingHints(null);
		qualityHints.put(RenderingHints.KEY_ALPHA_INTERPOLATION,
				RenderingHints.VALUE_ALPHA_INTERPOLATION_QUALITY);
		qualityHints.put(RenderingHints.KEY_RENDERING,
				RenderingHints.VALUE_RENDER_QUALITY);
		qualityHints.put(RenderingHints.KEY_ANTIALIASING,
				RenderingHints.VALUE_ANTIALIAS_ON);
		qualityHints.put(RenderingHints.KEY_INTERPOLATION,
				RenderingHints.VALUE_INTERPOLATION_BILINEAR);
		qualityHints.put(RenderingHints.KEY_COLOR_RENDERING,
				RenderingHints.VALUE_COLOR_RENDER_QUALITY);
		qualityHints.put(RenderingHints.KEY_DITHERING,
				RenderingHints.VALUE_DITHER_ENABLE);
	}

	private double rotation = -1.575;

	private BufferedImage image;

	public RotatingButton() {
		super();
		init();
	}

	public RotatingButton(Action action) {
		super(action);
		init();
	}

	public RotatingButton(Icon icon) {
		super(icon);
		init();
	}

	public RotatingButton(String text) {
		super(text);
		init();
	}

	public RotatingButton(String text, Icon icon) {
		super(text, icon);
		init();
	}

	@Override
	public Dimension getPreferredSize() {
		Dimension size = super.getPreferredSize();
		Area area = new Area(new Rectangle(size.width, size.height));
		area.transform(AffineTransform.getRotateInstance(rotation, size
				.getWidth() / 2.0, size.getHeight() / 2.0));
		Rectangle bounds = area.getBounds();
		size.setSize(bounds.width, bounds.height);
		return size;
	}

	public double getRotation() {
		return rotation;
	}

	@Override
	public void paint(Graphics g) {
		Graphics2D g2 = (Graphics2D) g;
		g2.setRenderingHints(qualityHints);
		g2.setColor(this.getBackground());
		g2.fillRect(0, 0, getWidth(), getHeight());
		g2.rotate(rotation, getWidth() / 2.0, getHeight() / 2.0);
		g2.drawImage(image, null, (getWidth() - image.getWidth()) / 2,
				(getHeight() - image.getHeight()) / 2);
	}

	public void setRotation(double rotation) {
		this.rotation = rotation;
		this.updateImage();
	}

	private void init() {
		updateImage();
		this.addPropertyChangeListener(new PropertyChangeListener() {

			@Override
			public void propertyChange(PropertyChangeEvent event) {
				updateImage();
			}
		});
		this.addChangeListener(new ChangeListener() {

			@Override
			public void stateChanged(ChangeEvent event) {
				updateImage();
			}
		});
	}

	private void updateImage() {
		this.setSize(super.getPreferredSize());
		image = new BufferedImage(getWidth(), getHeight(), BufferedImage.TYPE_INT_ARGB);
		Graphics2D ig = image.createGraphics();
		super.paint(ig);
		ig.dispose();
		this.setSize(this.getPreferredSize());
		if (this.getParent() instanceof JComponent) {
			((JComponent) this.getParent()).revalidate();
			((JComponent) this.getParent()).repaint();
		}
	}

}
